@@ -18,6 +18,7 @@ import (
 	testutils "github.com/openshift-kni/performance-addon-operators/functests/utils"
 	testclient "github.com/openshift-kni/performance-addon-operators/functests/utils/client"
 	"github.com/openshift-kni/performance-addon-operators/functests/utils/discovery"
+	"github.com/openshift-kni/performance-addon-operators/functests/utils/events"
 	"github.com/openshift-kni/performance-addon-operators/functests/utils/images"
 	testlog "github.com/openshift-kni/performance-addon-operators/functests/utils/log"
 	"github.com/openshift-kni/performance-addon-operators/functests/utils/nodes"
@@ -33,14 +34,15 @@ import (
 )
 
 const (
-	oslatTestName       = "oslat"
-	cyclictestTestName  = "cyclictest"
-	hwlatdetectTestName = "hwlatdetect"
-	defaultTestDelay    = 0
-	defaultTestRun      = false
-	defaultTestRuntime  = "300"
-	defaultMaxLatency   = -1
-	defaultTestCpus     = -1
+	oslatTestName        = "oslat"
+	cyclictestTestName   = "cyclictest"
+	hwlatdetectTestName  = "hwlatdetect"
+	defaultTestDelay     = 0
+	defaultTestRun       = false
+	defaultTestRuntime   = "300"
+	defaultMaxLatency    = -1
+	defaultTestCpus      = -1
+	minCpuAmountForOslat = 2
 )
 
 var (
@@ -97,7 +99,15 @@ var _ = Describe("[performance] Latency Test", func() {
 		Expect(err).ToNot(HaveOccurred(), "error looking for the optional selector: %v", err)
 
 		Expect(workerRTNodes).ToNot(BeEmpty())
-		workerRTNode = &workerRTNodes[0]
+
+		//At least one worker node should have cpu.Allocatable greater than the quantity requested by each test, else skip the test
+		workerRTNodesWithSufficientCpu := nodes.GetByCpuAllocatable(workerRTNodes, latencyTestCpus)
+		if len(workerRTNodesWithSufficientCpu) == 0 {
+			Skip("Insufficient cpu to run the test")
+
+		}
+		workerRTNode = &workerRTNodesWithSufficientCpu[0]
+
 	})
 
 	AfterEach(func() {
@@ -132,8 +142,11 @@ var _ = Describe("[performance] Latency Test", func() {
 			// do not calculated into the Allocated, at least part of time of one of isolated CPUs will be used to run
 			// other node containers
 			// at least two isolated CPUs to run oslat + one isolated CPU used by other containers on the node = at least 3 isolated CPUs
-			if isolatedCpus.Size() < 3 {
-				Skip(fmt.Sprintf("Skip the oslat test, the profile %q has less than two isolated CPUs", profile.Name))
+			if isolatedCpus.Size() < (minCpuAmountForOslat + 1) {
+				Skip(fmt.Sprintf("Skip the oslat test, the profile %q has less than %d isolated CPUs", profile.Name, minCpuAmountForOslat))
+			}
+			if latencyTestCpus < minCpuAmountForOslat && latencyTestCpus != defaultTestCpus {
+				Skip(fmt.Sprintf("Skip the oslat test, LATENCY_TEST_CPUS is less than the minimum CPUs amount %d", minCpuAmountForOslat))
 			}
 		})
 
@@ -164,7 +177,8 @@ var _ = Describe("[performance] Latency Test", func() {
 				Expect(curr < maximumLatency).To(BeTrue(), "The current latency %d is bigger than the expected one %d : \n %s", curr, maximumLatency, logFileContent)
 
 			}
-			testlog.Info(logFileContent)
+			//Use Println here so that this output will be displayed upon executing the test binary
+			fmt.Println(logFileContent)
 		})
 	})
 
@@ -206,7 +220,8 @@ var _ = Describe("[performance] Latency Test", func() {
 				Expect(curr < maximumLatency).To(BeTrue(), "The current latency %d is bigger than the expected one %d : \n %s", curr, maximumLatency, logFileContent)
 
 			}
-			testlog.Info(logFileContent)
+			//Use Println here so that this output will be displayed upon executing the test binary
+			fmt.Println(logFileContent)
 		})
 	})
 
@@ -243,8 +258,8 @@ var _ = Describe("[performance] Latency Test", func() {
 
 			// here we don't need to parse the latency values.
 			// hwlatdetect will do that for us and exit with error if needed.
-			testlog.Info(logFileContent)
-
+			//Use Println here so that this output will be displayed upon executing the test binary
+			fmt.Println(logFileContent)
 		})
 	})
 })
@@ -340,7 +355,7 @@ func getLatencyTestPod(profile *performancev2.PerformanceProfile, node *corev1.N
 	runnerName := fmt.Sprintf("%srunner", testNamePrefix)
 	runnerPath := path.Join("usr", "bin", runnerName)
 
-	if latencyTestCpus == -1 {
+	if latencyTestCpus == defaultTestCpus {
 		// we can not use all isolated CPUs, because if reserved and isolated include all node CPUs, and reserved CPUs
 		// do not calculated into the Allocated, at least part of time of one of isolated CPUs will be used to run
 		// other node containers
@@ -367,6 +382,7 @@ func getLatencyTestPod(profile *performancev2.PerformanceProfile, node *corev1.N
 			Annotations: map[string]string{
 				"irq-load-balancing.crio.io": "disable",
 				"cpu-load-balancing.crio.io": "disable",
+				"cpu-quota.crio.io":          "disable",
 			},
 			Namespace: testutils.NamespaceTesting,
 		},
@@ -416,6 +432,16 @@ func getLatencyTestPod(profile *performancev2.PerformanceProfile, node *corev1.N
 	}
 }
 
+func logEventsForPod(testPod *corev1.Pod) {
+	events, err := events.GetEventsForObject(testclient.Client, testPod.Namespace, testPod.Name, string(testPod.UID))
+	if err != nil {
+		testlog.Error(err)
+	}
+	for _, event := range events.Items {
+		testlog.Warningf("-> %s %s %s", event.Action, event.Reason, event.Message)
+	}
+}
+
 func createLatencyTestPod(testPod *corev1.Pod, node *corev1.Node, logName string) {
 	err := testclient.Client.Create(context.TODO(), testPod)
 	Expect(err).ToNot(HaveOccurred())
@@ -425,11 +451,28 @@ func createLatencyTestPod(testPod *corev1.Pod, node *corev1.Node, logName string
 
 	By("Waiting two minutes to download the latencyTest image")
 	err = pods.WaitForPhase(testPod, corev1.PodRunning, 2*time.Minute)
+	if err != nil {
+		testlog.Error(err)
+		logEventsForPod(testPod)
+	}
 	Expect(err).ToNot(HaveOccurred())
 
+	if runtime, _ := strconv.Atoi(latencyTestRuntime); runtime > 1 {
+		By("Checking actual CPUs number for the running pod")
+		limitsCpusQuantity := testPod.Spec.Containers[0].Resources.Limits.Cpu()
+		RequestsCpusQuantity := testPod.Spec.Containers[0].Resources.Requests.Cpu()
+		//latency pod is guaranteed
+		Expect(isEqual(limitsCpusQuantity, latencyTestCpus)).To(BeTrue(), fmt.Sprintf("actual limits of cpus number used for the latency pod is not as set in LATENCY_TEST_CPUS, actual number is: %s", limitsCpusQuantity))
+		Expect(isEqual(RequestsCpusQuantity, latencyTestCpus)).To(BeTrue(), fmt.Sprintf("actual requests of cpus number used for the latency pod is not as set in LATENCY_TEST_CPUS, actual number is: %s", RequestsCpusQuantity))
+	}
+
 	By("Waiting another two minutes to give enough time for the cluster to move the pod to Succeeded phase")
 	podTimeout := time.Duration(timeout + 120)
 	err = pods.WaitForPhase(testPod, corev1.PodSucceeded, podTimeout*time.Second)
+	if err != nil {
+		testlog.Error(err)
+		logEventsForPod(testPod)
+	}
 	Expect(err).ToNot(HaveOccurred(), getLogFile(node, logName))
 }
 
@@ -462,3 +505,7 @@ func removeLogfile(node *corev1.Node, logName string) {
 	}
 
 }
+
+func isEqual(qty *resource.Quantity, amount int) bool {
+	return qty.CmpInt64(int64(amount)) == 0
+}
